*-- File: DRVINFO.CC

   PROCEDURE GetDrvInfo (Drive)
   *-------------------------------------------------------------------
   *-- Programmer..: Marcello Perathoner
   *-- Date........: 12/12/1996
   *-- Notes.......: Allows "sugar syntax" calls to class DrvInfo like: 
   *--             : ? GetDrvInfo ("C").cSerial
   *-- Written for.: Visual dBASE 5.5
   *-- Rev. History: 12/12/1996 -- Original Code
   *-- Calls.......: hldbase.cc
   *-- Usage.......: oDI = GetDrvInfo (Drv)
   *-- Example.....: ? GetDrvInfo ("D").nTotalBytesOnDrive
   *--             : ? GetDrvInfo (3).cSerial
   *--             : inspect (GetDrvInfo (0))
   *-- Returns.....: object of class DrvInfo
   *-------------------------------------------------------------------

   RETURN new DrvInfo (Drive)
   *-- EoP: GetDrvInfo ()

CLASS DrvInfo (Drive) && "C" or "C:" or 0 = current, 1 = A, ...
*----------------------------------------------------------------------
*-- Description:
*
*      This class gets information about any drive in your system.
*      It accesses the disk.
*
*-- Programmer:
*
*      Marcello Perathoner (CIS: 100752,3200)
*
*-- History:
*
*      12/12/1996 -- original version of these routines
*
*-- Usage:
*
*      set procedure to drvinfo.cc additive
*      local oInfo; oInfo = new DrvInfo ("C")   && this "hits" the disk
*      ? oInfo.cVolume 
*      inspect (oInfo)
*
*-- Properties:
*     These properties are available only if the class was able to 
*     retrieve them. If there is no disk in the drive, the class
*     will have no properties. If the drive is a CDROM only the 
*     available info is retrieved. If any error is encountered
*     error will be set to .t.
*
*     To see what is there use :
*       inspect (new DrvInfo ("<DriveLetter>"))
*
*--   nDrive 
*       1 = A, 2 = B, ...
*
*--   cDrive 
*       "A", "B", ...
*
*---  lError
*       Set if anything did go wrong
*
*--   lLongFN
*       Set if drive (and OS) support long filenames
*
*--   lCaseSensitive
*       Set if filenames are case sensitive
*
*--   lCasePreserve
*       Set if filenames case is preserved
*
*--   lUniCode
*       Set if filenames are stored in unicode
*
*--   lVolumeCompressed
*       Set if filenames are stored in unicode
*
*--   cVolume 
*       Volume name as stored in the boot record
*
*--   cSerial
*       Volume serial nr as string 
*
*--   cFileSys
*       File System name: FAT16, FAT12, CD001, ... on Win31
*       FAT, CDFS, NTFS on Win95 / NT
*
*--   nSectorsPerCluster
*       How many sector make a cluster
*
*--   nAvailableClustres
*       How many clusters are available
*
*--   nBytesPerSector
*       How many bytes in a sector (mostly: 512, CD: 2048)
*
*--   nClustersPerDrive
*       How many clusters are on the drive
*
*--   nFreeBytesOnDrive
*       How many bytes are available
*
*--   nTotalBytesOnDrive
*       How big is the drive
*
*--   nReservedSectors
*       How many sectors are reserved for BIOS (usually 1)
*
*--   nSectorsPerFAT
*       How many sectors in a FAT
*
*--   nFATS
*       How many FATs on drive (usually 2, RAMDisks usually 1)
*
*--   nRootDirEntries
*       How many entries can you store in the root directory
*
*--   nFDirSecor
*       First Sector of root directory 
*
*--   nFUserSector
*       First Sector of user data (usually start of IO.SYS in DOS < 6)
*
*-- Dependencies:
*
*      HLDBASE.CC
*      HLDBASE.DLL
*----------------------------------------------------------------------

      #define DRIVE_REMOVABLE 2
      #define DRIVE_FIXED     3
      #define DRIVE_REMOTE    4

      extern CWORD WinAPIGetDriveType (CWORD) KERNEL from "GetDriveType"
      set procedure to hldbase.cc additive
      this.error = .f.
      local hl, out, in
      hl  = new HLDbase ()
      out = new regs ()    && create output regs

      this.nDrive = hl.DrvC2N (Drive)
      if this.nDrive = 0
         * get current drive
         in = new regs ()
         in.ah = htoi ("19")
         hl.intdos (in, out)
         this.nDrive = out.al + 1
      endif
      this.cDrive = chr (this.nDrive + asc ("A") - 1)
      
      in    = new regs ()  && use a new one for every call !!!
      in.ah = htoi ("36")  && dos function nr
      in.dl = this.nDrive    && drive nr
      
      hl.intdos (in, out)
      if out.ax <> htoi ("FFFF")
         this.nSectorsPerCluster = out.ax
         this.nAvailableClusters = out.bx
         this.nBytesPerSector    = out.cx
         this.nClustersPerDrive  = out.dx
         this.nFreeBytesOnDrive  = out.cx * out.ax * out.bx
         this.nTotalBytesOnDrive = out.cx * out.ax * out.dx
      else
         this.error = .t.
      endif
      
      in  = new regs ()
      hl.IOCTL (this.nDrive, htoi ("66"), in, out)
      if .not. out.carry
         this.cVolume  = trim (substr (in.dsdx, 7, 11))
         this.cVolume  = iif  (empty (this.cVolume), "No Label", this.cVolume)
         this.cSerial  = itoh (hlgets32 (in.dsdx, 3))
         this.cFileSys = trim (substr (in.dsdx, 18, 8))
      else
         this.error = .t.
      endif
      
      * This function returns a pointer to the drive parameter block
      * in dx:bx. To access ds:bx we must peek into memory.

      in = new regs ()		   && use a new one for every call !!!
      in.ah = htoi ("32")	   && function nr
      in.dl = this.nDrive            && drive nr
      hl.intdos (in, out)
      if out.al = 0

         * Now ds and bx point to the drive parameter block. We must copy
         * it into a string. hlpeek copies up to 32766 bytes into a string
         * starting at seg, off. Note that DOSMGR.VXD already converted 
         * the real mode pointer into a protected mode selector / offset
         * (but does this only for documented functions !).
      
         local cDPB
         cDPB = hl.peek (out.ds, out.bx, 33)  && copy 33 bytes at ds:bx to string

         * hlgetu8 gets an _U_nsigned 8 bits out of the string
         * hlgetu16 gets an _U_nsigned 16 bits out of the string

         this.nReservedSectors = hl.getu16 (cDPB,  7)
         this.nSectorsPerFAT   = hl.getu8  (cDPB, 16)
         this.nFATs            = hl.getu8  (cDPB,  9)
         this.nRootDirEntries  = hl.getu16 (cDPB, 10)
         this.nFDirSector      = hl.getu16 (cDPB, 18)
         this.nFUserSector     = hl.getu16 (cDPB, 12)
      else
         this.error = .t.
      endif

      local nDrvType
      nDrvType = WinAPIGetDriveType (this.nDrive - 1)
      this.cDrvType = "unknown"
      do case
         case nDrvType = DRIVE_REMOVABLE
            this.cDrvType = "removable"
         case nDrvType = DRIVE_FIXED
            this.cDrvType = "fixed"
         case nDrvType = DRIVE_REMOTE
            this.cDrvType = "remote"
      endcase

      * now the DOS70 stuff
      
      in = new regs ()
      in.ax = htoi ("71A0")
      hl.intdos (in, out)
      if out.ax = htoi ("7100")
         this.lLongFN        = .f.
         this.nMaxFileName   = 13
         this.nMaxPath       = 81
         this.lCaseSensitive = .f.
         this.lCasePreserved = .f. 
         this.lUniCode       = .f.
      else
      
         * VFAT.VXD detected
      
         in = new regs ()
         in.ax = htoi ("71A0")
         in.dsdx = this.cDrive + ":\"
         in.cx = 32
         in.esdi = hl.calloc (in.cx)
         hl.intdos (in, out)
         this.nMaxFileName   = out.cx + 1
         this.nMaxPath       = out.dx
         this.cFileSys       = hl.getstr (in.esdi, 1, 8)
         this.lLongFN        = bitset (out.bx, 14)
         this.lCaseSensitive = bitset (out.bx,  0)
         this.lCasePreserved = bitset (out.bx,  1)
         this.lUniCode       = bitset (out.bx,  2)
         this.lCompressed    = bitset (out.bx, 15)
     endif

ENDCLASS

*-- End of Class: DrvInfo

*-----------------------------------------------------------------------
*-- End of File: DRVINFO.CC
*-----------------------------------------------------------------------