*-- FILE: LOGCHECK.CC

CLASS LogCheck
*----------------------------------------------------------------------
*-- Description:
*
*      This CLASS is meant to monitor the login and logout of the users
*      of your application. It will use the ID() to store the users 
*      name, so be sure this works on your system if you want to find 
*      out who reboots in the middle of your apps. It stores the 
*      date/time and user name when instantiated in a file called 
*      LOGCHECK.DBF and completes it with the logout date and time when
*      released() Additionally it uses a timer to update a field called
*      LASTTIME at an interval you can specify in the code. It creates
*      a file called LASTLOG.DBF in the users C:\ directory and deletes
*      it on release(). So when this class is instantiated it can check
*      if this file exists and if this is the case, the user has 
*      rebooted when run the last time, and shows a message.
*
*-- Programmers:
*
*      Romain Strieff (CIS: 71333,2147)
*
*-- History: 
*
*      07/24/1996 -- Original
*      11/27/1996 -- original version of these routines as a class.
*
*-- Instantiation:
*
*      set procedure to logcheck.cc additive
*      _app.Log = new LogCheck()
*
*-- Methods:
*     Note, more details are in the headers for each method, including
*     a listing of parameters, and what they mean, as well as the
*     complete history (as we know it) for each method.
*
*--   Release()
*       Releases this from memory (and cleans out users's local
*       drive of tracking table)
*
*-- Properties:
*
*--   AlertUser
*       Logical -- alert user if they failed to exit properly?
*       Defaults to .t.
*
*-- WARNING:
*     This uses local SQL commands to create the tables used -- this 
*     means that when deploying an application you must deploy the SQL
*     portion of the BDE and/or the whole BDE ...
*----------------------------------------------------------------------

   this.AlertUser = .t.   && set to false if you don't want to alert 
                          && the user
   
   cAlias = Alias()
   select select()        && Next Work Area
    
   if this.AlertUser .and. file("c:\lastlog.dbf")
      use  c:\lastlog.dbf
      ?? CHR(7)
      msgbox("This Program has been rebooted on this machine"+chr(13)+;
             "by user <"+Trim(LoginId)+"> on the "+dtoc(Logindate) +;
             " at +- "+lasttime ,"REBOOTED!")
      use
      erase  c:\lastlog.dbf
   endif && this.AlertUser .and. file("c:\lastlog.db
   
   *-- if table doesn't exist, create it:
   *-- Move the table to the server when you are installing the tables 
   *-- only on the server and the .EXE on the local machine
   if .not. file ( "LOGCHECK.DBF" )
      create table "LOGCHECK.DBF" (  LOGCHECK.'LOGINID'     CHAR(25),;
        LOGCHECK.'LOGINDATE'   DATE, LOGCHECK.'LOGINTIME'   CHAR(5),;
        LOGCHECK.'LOGOUTDATE'  DATE, LOGCHECK.'LOGOUTTIME'  CHAR(5),;
        LOGCHECK.'LASTTIME'    CHAR(5))
   endif && .not. file ( "LOGCHECK.DBF" )
 
   select select()  && Next Work Area
   use LogCheck     && Open The Table

   *-- check if any SET FIELDS is active
   if set("FIELDS")="ON"
      *-- SET FIELDS so that VdB sees them
      set fields to all
   endif && Set("FIELDS")="ON"
 
   *-- add a new record
   append blank
 
   replace loginid with Id(), logindate with date(), ;
           logintime with time(), Lasttime with time()
   this.record = recno()
 
   use
   if .not. empty(cAlias)
      select (Calias)
   endif && .not. empty(cAlias)
 
   *-- Add a timer object which updates the user record with the 
   *-- current time so that we know approximately when he rebooted
   this.timer=new timer()   && Undocumented Object
   this.timer.interval=300  && seconds between  updates of the time
   this.timer.Parent=this   && TIMER objects need something to 
                            && reference this class
   this.timer.OnTimer={;this.parent.update()}  && call the Update 
                                               && method which sets the
                                               && time in the record
   this.Timer.enabled=.t.   &&enable the timer
 
   *---------------------Methods---------------------------------------
   FUNCTION Release
   *-- called before closing the application DO NOT FORGET THIS!!
      
      *--disable and remove the timer object
      this.timer.enabled=.f.
      this.timer=NULL
      *--normally all the tables should be closed but one never knows
      cAlias=Alias()
      select Select()  && Next Work Area
      use LogCheck     && Open The Table
      *-- check if any SET FIELDS is active
      if set("FIELDS")="ON"
         *-- SET FIELDS so that VdB sees them
         set fields to all
      endif && Set("FIELDS")="ON"
      n = this.record
      go n
      replace logoutdate with date(), logouttime with time()
     
      use
      if .not. empty(cAlias)
         select (cAlias)
      endif && .not. empty(cAlias)
      if file("c:\lastlog.dbf")
         erase c:\lastlog.dbf
      endif && file("c:\lastlog.dbf")
      release object this
   RETURN NULL
   
   *-------------------------------------------------------
   FUNCTION Update
      this.enabled=.f.    && THIS means the timer object
      *?? chr(7)          && uncomment for testing purposes
       
      cAlias=Alias()
      select Select()     && Next Work Area
      use LogCheck        && Open The Table
      *-- check if any SET FIELDS is active
      if set("FIELDS")="ON"
         *-- SET FIELDS so that VdB sees them
         set fields to all
      endif && Set("FIELDS")="ON"
      n = this.record
      go n
      replace lasttime with time()
      if file("c:\nul")
         copy next 1 to c:\lastlog.dbf
      endif && file("c:\nul")
      use
      if .not. empty(cAlias)
         select (cAlias)
      endif && .not. empty(cAlias)
      this.enabled=.t.
   RETURN NULL

ENDCLASS
*-- End of Class: LogCheck

*-----------------------------------------------------------------------
*-- End of File: LOGCHECK.CC
*-----------------------------------------------------------------------