*-- File: RECORD.CC

CLASS Record
*----------------------------------------------------------------------
*-- Description:
*
*      This class contains a set of record-processing routines.
*
*-- Programmers: 
*
*      Ken Mayer
*      Angus Scott-Fleming (CIS: 75500,3223)
*
*-- History: 
*
*      11/27/1996 -- original version of these routines as a class.
*
*-- Instantiation: 
*
*      set procedure to record.cc additive
*      _app.Record = new Record()
*
*-- Methods:
*      More details on each of these is contained in the commentary for
*      each method ...
*   
*--    Init()
*        Initializes classes necessary
*
*--    Release()
*        Releases this object (and others) from memory
*
*--    NewRec()
*        Recycles blank records
*
*--    RecChged()
*        Determines if a field has been changed
*
*-- Properties:
*
*      None
*----------------------------------------------------------------------


   FUNCTION Release
   *-------------------------------------------------------------------
   *-- Programmer..: Ken Mayer
   *-- Date........: 11/24/1996
   *-- Notes.......: Used to release this object from memory.
   *-- Written for.: Visual dBASE 5.5
   *-- Rev. History: 11/24/1996 -- Original Code
   *-- Calls.......: none
   *-- Usage.......: record.Release()
   *-- Example.....: _app.Record.Release()
   *--               _app.Record = NULL   && not required, but a good 
   *--                                    && idea
   *-- Returns.....: Null
   *-- Parameters..: None
   *-------------------------------------------------------------------

      release object this

   RETURN NULL
   *-- EoM: Release()

   FUNCTION NewRec
   *-------------------------------------------------------------------
   *-- Programmer..: Angus Scott-Fleming (CIS: 75500,3223)
   *-- Date........: 11/10/1996
   *-- Notes.......: This will recycle records instead of APPENDING 
   *--               Blank.  Effectively it recycles "deleted" records, 
   *--               but it requires that your routines that delete 
   *--               records actually blank them, and do NOT pack the 
   *--               database. If no blank records are in the database, 
   *--               the routine will append blank. Your delete 
   *--               routines should:
   *--                 BLANK the record and
   *--                 DELETE the record as well (turn on the delete 
   *--                        flag)
   *--               NOTE: This routine assumes you are using a 
   *--                     character field for your .MDX Tag Expression
   *-- Written for.: dBASE IV, 1.5 (will work with 1.1, but requires
   *--                            that you create a BLANK() routine...)
   *-- Rev. History: 05/28/1992 -- Original
   *--               05/03/1993 -- &cTagExpr. correction noted by Zak.
   *--               Fall, 1994 -- revised for dBASE 5.0 for Windows.
   *--                             Jay Parsons, 72662,1302
   *--               11/10/1996 -- Modified to work with this class.
   *-- Calls.......: None
   *-- Usage.......: record.NewRec()
   *-- Example.....: if oMyRecord.NewRec()
   *--                  replace fields with new data
   *--               else
   *--                  error routine
   *--               endif
   *-- Returns.....: Logical -- .T. if new record is locked, .F. if not
   *-- Parameters..: None
   *-------------------------------------------------------------------
      
      local lOldExact, cTagExpr
      
      lOldExact = set("EXACT")="ON"
      set exact on
      set deleted off
      
      *-- get tag expression for current .MDX tag
      cTagExpr = key(tagno(order()))
      
      *-- look for a blank record, and attempt to lock the record
      if seek(space(len(&cTagExpr.))) .and. rlock()
         recall                       && turn off deleted flag
      else                            && no blank records
         append blank                 && add one
      endif
      
      *-- reset
      set deleted on
      if .not. lOldExact
         set exact off
      endif
      
   RETURN rlock()
   *-- EoM: NewRec()

   FUNCTION RecChged( cTable_Name )
   *-------------------------------------------------------------------
   *-- Programmer..: Angus Scott-Fleming (CIS: 76711,617)
   *-- Date........: 11/10/1996
   *-- Notes.......: Test field values against memory variables to see 
   *--               if an on-screen display has changed from the disk-
   *--               record CHANGE() requires the existence of field 
   *--               _DBASELOCK whereas RecChged does not.
   *-- Written for.: dBASE IV 1.1+
   *-- Rev. History: 11/25/1992 -- for dBase IV 1.5
   *--               10/08/1992 -- don't test memo-fields
   *--               06/09/1992 -- dropped PCount() for 4.11 use
   *--               06/04/1992 -- skips _DBASELOCK field
   *--               08/02/1993 -- minor tuning
   *--               Fall, 1994 -- revised for dBASE 5.0 for Windows.
   *--                             Jay Parsons, 72662,1302
   *--               11/10/1996 -- Modified to work with this class.
   *-- Calls.......: None
   *-- Usage.......: record.RecChged(<cTable_Name>)
   *-- Example.....: if oMyRecord.RecChged("mpl") .and. ;
   *--                  Confirm("Save?",.Y.)
   *-- Returns.....: .T. = record is changed  .F. = record is not 
   *--                     changed
   *-- Parameters..: cTable_Name = (OPTIONAL) alias of table to test
   *-------------------------------------------------------------------
      
      local n, test_field, test_disk
      
      if empty(ctable_name)
         ctable_name = alias()
      endif
      for n = 1 to fldcount(ctable_name)
          test_field = field(n,ctable_name)
          test_disk = "&ctable_name.->&test_field."
          * Thu  06-04-1992  don't test _DBASELOCK field
          * Thu  10-08-1992  check for existence of the field in the
          * table skip check for memo fields
          * 11/10/1996 -- changed call to 'EXEQUAL' to use
          * "==" operator ...
          if .not. upper(test_field) = "_DBASELOCK" .and. ;
             .not. type("&test_disk.") $ "MU" .and. ;
             .not. type("&test_field.") ="U" .and. ;
             .not. (&test_field. == &test_disk.)
             RETURN .T.
          endif
      endfor
      
   RETURN .F.
   *-- EoM: RecChged()

ENDCLASS
*-- End of Class: RECORD

*-----------------------------------------------------------------------
*-- End of File: RECORD.CC
*-----------------------------------------------------------------------