*-- FILE: WINATTR.CC

CLASS WINATTR(oForm)
      && Pass in the form instance (after the form is open)
*----------------------------------------------------------------------
*-- Description:
*
*      This UI file is designed to allow you to change attributes of
*      a form (setting the maximize, minimize and other properties ...)
*
*-- Programmers: 
*
*      David Wedwick (CIS: 73677, 2600)
*
*-- History: 
*
*      12/03/1996 -- First version of this custom class file for dUFLP
*                    library.
*
*-- Instantiation: 
*
*     * in the form's OnOpen (or over-riden open/readmodal event)
*     set procedure to winattr.cc additive
*     this.Attr = new winattr(form)
*
*-- Note:
*      The form must have an hWND value, so that this can be called
*      in the form's OnOpen (or whatever).
*
*-- Methods:
*
*--    MinButton( <lNew> )
*        Changes minimize to new setting
*
*--    MaxButton( <lNew> )
*        Same for Maximize
*
*--    DlgFrame( <lNew> )
*        Same for Dialog Frame
*
*--    Border( <lNew> )
*        Same for border
*
*--    ThickFrame( <lNew> )
*        Same for 'thick frame'
*
*--    SysMenu( <lNew> )
*        SysMenu box in upper left corner ...
*----------------------------------------------------------------------
   
   PROTECT Setting
   
   if type("GetWindowLong") <> "FP"
      EXTERN CLONG GetWindowLong(CHANDLE, CINT) USER
   endif
   
   if type("SetWindowLong") <> "FP"
      EXTERN CLONG SetWindowLong(CHANDLE, CINT, CLONG) USER
   endif
   
   if type("SetWindowPos") <> "FP"
      EXTERN CLOGICAL SetWindowPos(CHANDLE, CHANDLE, CINT, CINT, CINT,;
                                   CINT, CINT) USER
   endif
   
   *-- Save a reference to the form's handle
   this.formhWND = oForm.hWND
   
   *-- Retrieve the current STYLE settings for the form
   this.nStyle = GetWindowLong(this.formhWND, -16)
   
   FUNCTION MinButton(lNewValue)
      *-- RETURN .t. if the MinButton is enabled on the form
      *-- or .f. if not.  If lNewValue is passed, try to change the 
      *-- setting based on the new value.  RETURN the value before any 
      *-- changes.
      *-- Check the position in bit 17
   RETURN this.Setting(17, pcount() == 1, lNewValue)
   
   FUNCTION MaxButton(lNewValue)
      *-- RETURN .t. if the MaxButton is enabled on the form
      *-- or .f. if not.  If lNewValue is passed, try to change the 
      *-- setting based on the new value.  RETURN the value before any 
      *-- changes.
      *-- Check the position in bit 16
   RETURN this.Setting(16, pcount() == 1, lNewValue)

   FUNCTION DlgFrame(lNewValue)
      *-- RETURN .t. if the DlgFrame flag is enabled on the form
      *-- or .f. if not.  If lNewValue is passed, try to change the 
      *-- setting based on the new value.  RETURN the value before any
      *-- changes.
      *-- Check the position in bit 22
   RETURN this.Setting(22, pcount() == 1, lNewValue)

   FUNCTION Border(lNewValue)
      *-- RETURN .t. if the Border flag is enabled on the form
      *-- or .f. if not.  If lNewValue is passed, try to change the 
      *-- setting based on the new value.  RETURN the value before any 
      *-- changes.
      *-- Check the position in bit 23
   RETURN this.Setting(23, pcount() == 1, lNewValue)

   FUNCTION ThickFrame(lNewValue)
      *-- RETURN .t. if the ThickFrame flag is enabled on the form
      *-- or .f. if not.  If lNewValue is passed, try to change the 
      *-- setting based on the new value.  RETURN the value before any 
      *-- changes.
      *-- Check the position in bit 18
   RETURN this.Setting(18, pcount() == 1, lNewValue)

   FUNCTION SysMenu(lNewValue)
      *-- RETURN .t. if the SysMenu flag is enabled on the form
      *-- or .f. if not.  If lNewValue is passed, try to change the 
      *-- setting based on the new value.  RETURN the value before any 
      *-- changes.
      *-- Check the position in bit 19
   RETURN this.Setting(19, pcount() == 1, lNewValue)

  FUNCTION Setting(nBitPos, lChange, lNew)
    *-- RETURN the current setting in nBitPos of the form's STYLE 
    *-- setting. If lChange is .t., change the setting to reflect the 
    *-- value in lNew.
    LOCAL lSet, nMask

    lSet = bitset(this.nStyle, nBitPos)

    if lChange
      nMask = bitlshift(1, nBitPos)

      if lNew .and. .not. lSet
        *-- Set the flag
        this.nStyle = this.nStyle + nMask
      elseif .not. lNew .and. lSet
        *-- Remove the flag
        this.nStyle = this.nStyle - nMask
      endif

      *-- Change the setting
      SetWindowLong(this.formhWND, -16, this.nStyle)

      *-- Make the changes visible.  This code kindly provided by 
      *-- Romain Strieff.
      *--    Thanks!
      #define SWP_NOSIZE          1
      #define SWP_NOMOVE          2
      #define SWP_FRAMECHANGED   32
      SetWindowPos(this.formhWND, 0,0,0,0,0, SWP_FRAMECHANGED+;
                   SWP_NOMOVE+SWP_NOSIZE)

      *-- Retrieve the new setting (should be the same, but if not...)
      this.nStyle = GetWindowLong(this.formhWND, -16)
    endif

    RETURN lSet

ENDCLASS
*-- End of Class: WinAttr

*-----------------------------------------------------------------------
*-- End of File: WINATTR.CC
*-----------------------------------------------------------------------
